searchData={"content_type":"text/markdown","items":[{"doc":"LiveQuery allows you to declare how to load, invalidate, and reload your data independently from your consumer logic.\nAn example is going to help a lot here.\n\nImagine you have a list of users in your database, and one or more consumers in your system want to \"use\" this list of users.\nI'm saying \"use\" to because it could mean a lot of things.\nMaybe you want to display the list of users in live view.\nMaybe you want return the list as a response to an API call.\nMaybe you just want to log them.\nLiveQuery doesn't care.\n\nRegardless, before you can consume a query you must define it.\nA query definition (def) can take a lot of forms, but for now we'll just write it in a module.\n\n```elixir\ndefmodule Users do\n  use LiveQuery.Query.Def\n\n  @impl true\n  def init(_ctx) do\n    Repo.all(User)\n  end\nend\n```\n\nThis is enough to get started, but since we want a \"live\" user's list we'll also need to define how to subscribe to updates and reload our list in response to those updates.\n\n```elixir\ndefmodule Demo.Users do\n  use LiveQuery.Query.Def\n\n  @impl true\n  def init(_ctx) do\n    Phoenix.PubSub.subscribe(Demo.PubSub, \"users\")\n    Repo.all(User)\n  end\n\n  @impl true\n  def handle_info(_msg, _from, _ctx) do\n    Repo.all(User)\n  end\nend\n```\n\nAgain, notice that this definition is consumer agnostic.\nIt says how to load a live user's list, but it doesn't care how you use it.\nSo, now let's use it.\nHow about from a Phoenix controller?\n\n```elixir\ndef DemoWeb.UsersController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    LiveQuery.link(Demo.LiveQuery,\n      query_key: [:users],\n      query_def: Demo.Users,\n      query_config: %{},\n      client_pid: self()\n    )\n\n    users = LiveQuery.read(Demo.LiveQuery, query_key: [:users]).value\n    render(conn, :index, users: users)\n  end\nend\n```\n\nThis will almost work, but we need to make one more change.\nLiveQuery actually runs your queries for you, so you need to start it as part of your application.\n\n```elixir\ndef Demo.Supervisor do\n  ...\n\n  @impl true\n  def init(_opts) do\n    children = [\n      ...\n      {LiveQuery, name: Demo.LiveQuery},\n      ...\n    ]\n\n    ...\n  end\nend\n```\n\nNow when someone visits the users index page, and the controller's index action is invoked our users list will be returned.\nWhat's cool is that this list will only exist at most once in memory at any given time.\nSo, if lots of people visit the users index page at the same time, we don't have to worry about excess load on our database, or excess memory usage in our application.\n\nNow, you might say, this is fine, but it seems like the controller is doing a lot of work just to get a list of users.\nThis is where LiveQuery client libraries come in.\nLiveQuery is consumer agnostic which is part of what makes it so powerful. You can write a query definition once, and then use it anywhere.\nHowever, it also means that you have to write a lot of boilerplate when you consume queries.\nClient libraries hide this boilerplate behind a simple API designed for a given consumer.\n\nUsing LiveQuery.Client (coming soon) we can rewrite our controller like this:\n\n```elixir\ndef DemoWeb.UsersController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    users = LiveQuery.Client.fetch!(Demo.LiveQuery, query_def: Demo.Users)\n    render(conn, :index, users: users)\n  end\nend\n```\n\nThis is much simpler, and you still get all the reuse and generalization benefits of LiveQuery.\n\nHopefully, this example has given you a taste of what LiveQuery can do.\nYou can explore individual function and module documentation for more details.","ref":"LiveQuery.html","title":"LiveQuery","type":"module"},{"doc":"Used when running LiveQuery underneath a supervisor. Works the same way GenServer.child_spec/1 does.","ref":"LiveQuery.html#child_spec/1","title":"LiveQuery.child_spec/1","type":"function"},{"doc":"Link a client process to a query.\nIf the query process is not running, it will be started, but not initialized (that only happens once a client tries to read the query).\nOnce at least one client process is linked to a query, you can rely on the query existing until all client processes are unlink from it.\nIf you link a client process to a query and the client processes dies, the query will be unlinked from the client process automatically.\n\n- `:name` - the name of the LiveQuery system\n- `:opts` - options concerning the query being linked\n  - `:query_key` - the key used to identify the query in the LiveQuery system\n  - `:query_def` - the query definition (can be anything that implements the `LiveQuery.Query.DefLike` protocol)\n  - `:query_config` - configuration for the query\n  - `:client_pid` - the pid of the process that is linking to the query (usually yourself)","ref":"LiveQuery.html#link/2","title":"LiveQuery.link/2","type":"function"},{"doc":"Read the value of a query.\nIf the query is not running, a `LiveQuery.Protocol.NoQuery` struct will be returned.\nIf the query is running, a `LiveQuery.Protocol.Data` struct will be returned.\nIf the query hasn't yet been initialized, it will be initialized before returning the value.\n\n- `:name` - the name of the LiveQuery system\n- `:opts` - options concerning the query being linked\n  - `:query_key` - the key used to identify the query in the LiveQuery system\n  - `:selector` - the function used to transform the query's value before returning it","ref":"LiveQuery.html#read/2","title":"LiveQuery.read/2","type":"function"},{"doc":"Register a callback to be invoked with a `LiveQuery.Protocol.DataChanged` struct when a query's value changes.\nIf the query is not running, a `LiveQuery.Protocol.NoQuery` struct will be returned.\nCallbacks allow consumers to inject logic into the query's lifecycle.\nCommonly this is used to have the query notify the consumer when it's value changes.\nAll registered callbacks are automatically unregistered when their client process is unlinked.\n\n- `:name` - the name of the LiveQuery system\n- `:opts` - options concerning the query being linked\n  - `:query_key` - the key used to identify the query in the LiveQuery system\n  - `:client_pid` - the client process that is registering the callback (usually yourself)\n  - `:cb_key` - uniquely identifies the callback so it can later be removed\n  - `:cb` - the function to be invoked when the query's value changes","ref":"LiveQuery.html#register_callback/2","title":"LiveQuery.register_callback/2","type":"function"},{"doc":"Starts a LiveQuery.\n\n- `:opts` - options concerning the query being linked\n  - `:name` - the name of the LiveQuery system","ref":"LiveQuery.html#start_link/1","title":"LiveQuery.start_link/1","type":"function"},{"doc":"Unlink a client process from a query.\nIf the query has no more clients, it will be stopped.\nUnlinking a client process from a query that it is not already linked to is a no-op.\n\n- `:name` - the name of the LiveQuery system\n- `:opts` - options concerning the query being linked\n  - `:query_key` - the key used to identify the query in the LiveQuery system\n  - `:client_pid` - the pid of the process that is linking to the query (usually yourself)","ref":"LiveQuery.html#unlink/2","title":"LiveQuery.unlink/2","type":"function"},{"doc":"Unregisters all callbacks from a query for a given client process.\nIdentical to calling `unregister_callback/2` for all callbacks that have been registered for a given client process.\n\n- `:name` - the name of the LiveQuery system\n- `:opts` - options concerning the query being linked\n  - `:query_key` - the key used to identify the query in the LiveQuery system\n  - `:client_pid` - the client process that is registering the callback (usually yourself)","ref":"LiveQuery.html#unregister_all_callbacks/2","title":"LiveQuery.unregister_all_callbacks/2","type":"function"},{"doc":"Unregister a callback from a query.\nIf the query is not running, a `LiveQuery.Protocol.NoQuery` struct will be returned.\n\n- `:name` - the name of the LiveQuery system\n- `:opts` - options concerning the query being linked\n  - `:query_key` - the key used to identify the query in the LiveQuery system\n  - `:client_pid` - the client process that is registering the callback (usually yourself)\n  - `:cb_key` - uniquely identifies the callback (passed in when the query was registered)","ref":"LiveQuery.html#unregister_callback/2","title":"LiveQuery.unregister_callback/2","type":"function"},{"doc":"Sent in response to a `LiveQuery.Protocol.RegisterCallback` call.\nThe callback's return value is discarded by the query.\nThe `cb_key` is used to identify the callback when unregistering it. It is namespaced to the `client_pid`.\nThat is, different clients can register callbacks with the same `cb_key` at the same time and they will not interfere with each other.","ref":"LiveQuery.Protocol.CallbackRegistered.html","title":"LiveQuery.Protocol.CallbackRegistered","type":"module"},{"doc":"Sent in response to a `LiveQuery.Protocol.UnegisterCallback` call.\nAdditionally, a list of these are returned in response to a `LiveQuery.Protocol.UnegisterAllCallbacks` call.\nThe `cb_key` is the same one used to identify the callback when registering it.","ref":"LiveQuery.Protocol.CallbackUnregistered.html","title":"LiveQuery.Protocol.CallbackUnregistered","type":"module"},{"doc":"Returned in response to a `LiveQuery.Protocol.Read` call (when the query exists).","ref":"LiveQuery.Protocol.Data.html","title":"LiveQuery.Protocol.Data","type":"module"},{"doc":"Given to a callback when the data for a query changes.","ref":"LiveQuery.Protocol.DataChanged.html","title":"LiveQuery.Protocol.DataChanged","type":"module"},{"doc":"Link a client process to a query.\nOnce linked to a query, the client will be monitored, and the query will be guaranteed to exist so long as the client is alive and has manually unlinked from the query.","ref":"LiveQuery.Protocol.Link.html","title":"LiveQuery.Protocol.Link","type":"module"},{"doc":"Create a new `LiveQuery.Protocol.Link` struct safely.\nThis validates the created struct will be considered valid by LiveQuery.\nPrefer using this function over literal struct creation when possible.","ref":"LiveQuery.Protocol.Link.html#new/1","title":"LiveQuery.Protocol.Link.new/1","type":"function"},{"doc":"Returned in response to a `LiveQuery.Protocol.Link` call.","ref":"LiveQuery.Protocol.Linked.html","title":"LiveQuery.Protocol.Linked","type":"module"},{"doc":"Returned whenever a operation is attepted against a query that doesn't exist.\nThe only exception is `LiveQuery.Protocol.Link` and `LiveQuery.Protocol.Unlink`.","ref":"LiveQuery.Protocol.NoQuery.html","title":"LiveQuery.Protocol.NoQuery","type":"module"},{"doc":"Read the value of a query (the query's init function will be called if it hasn't been called yet).\nThe `selector` function will be called with the query's value and before the result is returned.","ref":"LiveQuery.Protocol.Read.html","title":"LiveQuery.Protocol.Read","type":"module"},{"doc":"Create a new `LiveQuery.Protocol.Read` struct.\nIf `selector` is not provided the indentity function will be used by default.","ref":"LiveQuery.Protocol.Read.html#new/1","title":"LiveQuery.Protocol.Read.new/1","type":"function"},{"doc":"TODO","ref":"LiveQuery.Protocol.RegisterCallback.html","title":"LiveQuery.Protocol.RegisterCallback","type":"module"},{"doc":"TODO","ref":"LiveQuery.Protocol.RegisterCallback.html#new/1","title":"LiveQuery.Protocol.RegisterCallback.new/1","type":"function"},{"doc":"TODO","ref":"LiveQuery.Protocol.Unlink.html","title":"LiveQuery.Protocol.Unlink","type":"module"},{"doc":"TODO","ref":"LiveQuery.Protocol.Unlink.html#new/1","title":"LiveQuery.Protocol.Unlink.new/1","type":"function"},{"doc":"TODO","ref":"LiveQuery.Protocol.Unlinked.html","title":"LiveQuery.Protocol.Unlinked","type":"module"},{"doc":"TODO","ref":"LiveQuery.Protocol.UnregisterAllCallbacks.html","title":"LiveQuery.Protocol.UnregisterAllCallbacks","type":"module"},{"doc":"TODO","ref":"LiveQuery.Protocol.UnregisterAllCallbacks.html#new/1","title":"LiveQuery.Protocol.UnregisterAllCallbacks.new/1","type":"function"},{"doc":"TODO","ref":"LiveQuery.Protocol.UnregisterCallback.html","title":"LiveQuery.Protocol.UnregisterCallback","type":"module"},{"doc":"TODO","ref":"LiveQuery.Protocol.UnregisterCallback.html#new/1","title":"LiveQuery.Protocol.UnregisterCallback.new/1","type":"function"},{"doc":"This module defines the `LiveQuery.Query.Def` behaviour and struct.\nThe behaviour can be used to make a module-base query definition.\nThe struct can be used as a query definition itself.","ref":"LiveQuery.Query.Def.html","title":"LiveQuery.Query.Def","type":"behaviour"},{"doc":"Just adds the `LiveQuery.Query.Def` behaviour to your module for you.\nModules which implement this behaviour can be used as a query definition.","ref":"LiveQuery.Query.Def.html#__using__/1","title":"LiveQuery.Query.Def.__using__/1","type":"macro"},{"doc":"","ref":"LiveQuery.Query.Def.html#c:handle_call/3","title":"LiveQuery.Query.Def.handle_call/3","type":"callback"},{"doc":"","ref":"LiveQuery.Query.Def.html#c:handle_cast/2","title":"LiveQuery.Query.Def.handle_cast/2","type":"callback"},{"doc":"","ref":"LiveQuery.Query.Def.html#c:handle_info/2","title":"LiveQuery.Query.Def.handle_info/2","type":"callback"},{"doc":"","ref":"LiveQuery.Query.Def.html#c:init/1","title":"LiveQuery.Query.Def.init/1","type":"callback"},{"doc":"Creates a new `LiveQuery.Query.Def` struct which implements the `LiveQuery.Query.DefLike` protocol.","ref":"LiveQuery.Query.Def.html#new/1","title":"LiveQuery.Query.Def.new/1","type":"function"},{"doc":"","ref":"LiveQuery.Query.Def.html#t:data/0","title":"LiveQuery.Query.Def.data/0","type":"type"},{"doc":"","ref":"LiveQuery.Query.Def.html#t:t/0","title":"LiveQuery.Query.Def.t/0","type":"type"},{"doc":"A query definition defines how to load and maintain a query.\nIt's the blueprint of a query.\nAnything that implements the `LiveQuery.Query.DefLike` protocol can be used as a query definition.","ref":"LiveQuery.Query.DefLike.html","title":"LiveQuery.Query.DefLike","type":"protocol"},{"doc":"Your query process is like a GenServer.\nYou can handle calls, but you're limited in what you can return since queries don't allow for handle_continue.","ref":"LiveQuery.Query.DefLike.html#handle_call/4","title":"LiveQuery.Query.DefLike.handle_call/4","type":"function"},{"doc":"Your query process is like a GenServer.\nYou can handle casts, but you're limited in what you can return since queries don't allow for handle_continue.\nYou must return your query's new value.","ref":"LiveQuery.Query.DefLike.html#handle_cast/3","title":"LiveQuery.Query.DefLike.handle_cast/3","type":"function"},{"doc":"Your query process is like a GenServer.\nYou can handle messages, but you're limited in what you can return since queries don't allow for handle_continue.\nYou must return your query's new value.","ref":"LiveQuery.Query.DefLike.html#handle_info/3","title":"LiveQuery.Query.DefLike.handle_info/3","type":"function"},{"doc":"Called when the query is initialized.\nThis is where the query should be loaded and any subscriptions should be setup.","ref":"LiveQuery.Query.DefLike.html#init/2","title":"LiveQuery.Query.DefLike.init/2","type":"function"},{"doc":"","ref":"LiveQuery.Query.DefLike.html#t:data/0","title":"LiveQuery.Query.DefLike.data/0","type":"type"},{"doc":"","ref":"LiveQuery.Query.DefLike.html#t:t/0","title":"LiveQuery.Query.DefLike.t/0","type":"type"}]}